"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const SwipeableRow_1 = require("./SwipeableRow");
/**
 * A container component that renders multiple SwipeableRow's in a FlatList
 * implementation. This is designed to be a drop-in replacement for the
 * standard React Native `FlatList`, so use it as if it were a FlatList, but
 * with extra props, i.e.
 *
 * <SwipeableListView renderRow={..} renderQuickActions={..} {..FlatList props} />
 *
 * SwipeableRow can be used independently of this component, but the main
 * benefit of using this component is
 *
 * - It ensures that at most 1 row is swiped open (auto closes others)
 * - It can bounce the 1st row of the list so users know it's swipeable
 * - Increase performance on iOS by locking list swiping when row swiping is occurring
 * - More to come
 */
exports.SwipeableFlatList = (props) => {
    const _flatListRef = react_1.useRef();
    const [openRowKey, setOpenRowKey] = react_1.useState(null);
    const _onScroll = (e) => {
        // Close any opens rows on ListView scroll
        if (openRowKey !== null) {
            setOpenRowKey(null);
        }
        props.onScroll && props.onScroll(e);
    };
    const _renderItem = (info) => {
        const { renderQuickActions = () => null } = props;
        const slideoutView = renderQuickActions(info);
        const key = props.keyExtractor
            ? props.keyExtractor(info.item, info.index)
            : info.index;
        // If renderQuickActions is unspecified or returns falsey, don't allow swipe
        if (!slideoutView) {
            return props.renderItem ? props.renderItem(info) : null;
        }
        return (react_1.default.createElement(SwipeableRow_1.SwipeableRow, { slideoutView: slideoutView, isOpen: key === openRowKey, maxSwipeDistance: _getMaxSwipeDistance(info), onOpen: () => _onOpen(key), onClose: () => _onClose(key), shouldBounceOnMount: props.bounceFirstRowOnMount, onSwipeEnd: _setListViewScrollable, onSwipeStart: _setListViewNotScrollable }, props.renderItem && props.renderItem(info)));
    };
    // This enables rows having variable width slideoutView.
    const _getMaxSwipeDistance = (info) => {
        if (typeof props.maxSwipeDistance === 'function') {
            return props.maxSwipeDistance(info);
        }
        return props.maxSwipeDistance || 50;
    };
    const _setListViewScrollableTo = (value) => {
        if (_flatListRef.current) {
            _flatListRef.current.setNativeProps({
                scrollEnabled: value,
            });
        }
    };
    const _setListViewScrollable = () => {
        _setListViewScrollableTo(true);
    };
    const _setListViewNotScrollable = () => {
        _setListViewScrollableTo(false);
    };
    const _onOpen = (key) => {
        setOpenRowKey(key);
    };
    const _onClose = (_key) => {
        setOpenRowKey(null);
    };
    return (react_1.default.createElement(react_native_1.FlatList, Object.assign({}, props, { ref: ref => {
            if (ref !== null) {
                _flatListRef.current = ref;
            }
        }, onScroll: _onScroll, renderItem: _renderItem })));
};
