"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const IS_RTL = react_native_1.I18nManager.isRTL;
// NOTE: Eventually convert these consts to an input object of configurations
// Position of the left of the swipable item when closed
const CLOSED_LEFT_POSITION = 0;
// Minimum swipe distance before we recognize it as such
const HORIZONTAL_SWIPE_DISTANCE_THRESHOLD = 10;
// Minimum swipe speed before we fully animate the user's action (open/close)
const HORIZONTAL_FULL_SWIPE_SPEED_THRESHOLD = 0.3;
// Factor to divide by to get slow speed; i.e. 4 means 1/4 of full speed
const SLOW_SPEED_SWIPE_FACTOR = 4;
// Time, in milliseconds, of how long the animated swipe should be
const SWIPE_DURATION = 300;
/**
 * On SwipeableListView mount, the 1st item will bounce to show users it's
 * possible to swipe
 */
const ON_MOUNT_BOUNCE_DELAY = 700;
const ON_MOUNT_BOUNCE_DURATION = 400;
// Distance left of closed position to bounce back when right-swiping from closed
const RIGHT_SWIPE_BOUNCE_BACK_DISTANCE = 30;
const RIGHT_SWIPE_BOUNCE_BACK_DURATION = 300;
/**
 * Max distance of right swipe to allow (right swipes do functionally nothing).
 * Must be multiplied by SLOW_SPEED_SWIPE_FACTOR because gestureState.dx tracks
 * how far the finger swipes, and not the actual animation distance.
 */
const RIGHT_SWIPE_THRESHOLD = 30 * SLOW_SPEED_SWIPE_FACTOR;
const emptyFunction = () => { };
/**
 * Creates a swipable row that allows taps on the main item and a custom View
 * on the item hidden behind the row. Typically this should be used in
 * conjunction with SwipeableListView for additional functionality, but can be
 * used in a normal ListView. See the renderRow for SwipeableListView to see how
 * to use this component separately.
 */
exports.SwipeableRow = ({ children, isOpen = false, maxSwipeDistance = 0, onClose = emptyFunction, onOpen = emptyFunction, onSwipeEnd = emptyFunction, onSwipeStart = emptyFunction, preventSwipeRight = false, shouldBounceOnMount, slideoutView, swipeThreshold = 30, }) => {
    const [isSwipeableViewRendered, setSwipeableViewRendered] = react_1.useState(false);
    const [rowHeight, setRowHeight] = react_1.useState(0);
    const _previousLeft = react_1.useRef(0);
    const currentLeft = new react_native_1.Animated.Value(_previousLeft.current);
    const _animateTo = react_1.useCallback((toValue, duration = SWIPE_DURATION, callback = emptyFunction) => {
        react_native_1.Animated.timing(currentLeft, {
            duration,
            toValue,
            useNativeDriver: true,
        }).start(() => {
            _previousLeft.current = toValue;
            callback();
        });
    }, [currentLeft]);
    const _animateToOpenPositionWith = react_1.useCallback((speed, distMoved) => {
        /**
         * Ensure the speed is at least the set speed threshold to prevent a slow
         * swiping animation
         */
        speed =
            speed > HORIZONTAL_FULL_SWIPE_SPEED_THRESHOLD
                ? speed
                : HORIZONTAL_FULL_SWIPE_SPEED_THRESHOLD;
        /**
         * Calculate the duration the row should take to swipe the remaining distance
         * at the same speed the user swiped (or the speed threshold)
         */
        const duration = Math.abs((maxSwipeDistance - Math.abs(distMoved)) / speed);
        const distance = IS_RTL ? -maxSwipeDistance : maxSwipeDistance;
        _animateTo(-distance, duration);
    }, [_animateTo, maxSwipeDistance]);
    const _shouldAnimateRemainder = react_1.useCallback((gestureState) => {
        /**
         * If user has swiped past a certain distance, animate the rest of the way
         * if they let go
         */
        return (Math.abs(gestureState.dx) > swipeThreshold ||
            gestureState.vx > HORIZONTAL_FULL_SWIPE_SPEED_THRESHOLD);
    }, [swipeThreshold]);
    const _animateToClosedPosition = react_1.useCallback((duration = SWIPE_DURATION) => {
        _animateTo(CLOSED_LEFT_POSITION, duration);
    }, [_animateTo]);
    const _animateToClosedPositionDuringBounce = react_1.useCallback(() => {
        _animateToClosedPosition(RIGHT_SWIPE_BOUNCE_BACK_DURATION);
    }, [_animateToClosedPosition]);
    const _swipeFullSpeed = react_1.useCallback((gestureState) => {
        currentLeft.setValue(_previousLeft.current + gestureState.dx);
    }, [currentLeft]);
    const _swipeSlowSpeed = react_1.useCallback((gestureState) => {
        currentLeft.setValue(_previousLeft.current + gestureState.dx / SLOW_SPEED_SWIPE_FACTOR);
    }, [currentLeft]);
    const _isSwipingRightFromClosed = react_1.useCallback((gestureState) => {
        const gestureStateDx = IS_RTL ? -gestureState.dx : gestureState.dx;
        return (_previousLeft.current === CLOSED_LEFT_POSITION && gestureStateDx > 0);
    }, []);
    const _isSwipingExcessivelyRightFromClosedPosition = react_1.useCallback((gestureState) => {
        /**
         * We want to allow a BIT of right swipe, to allow users to know that
         * swiping is available, but swiping right does not do anything
         * functionally.
         */
        const gestureStateDx = IS_RTL ? -gestureState.dx : gestureState.dx;
        return (_isSwipingRightFromClosed(gestureState) &&
            gestureStateDx > RIGHT_SWIPE_THRESHOLD);
    }, [_isSwipingRightFromClosed]);
    const _handlePanResponderMove = react_1.useCallback((_event, gestureState) => {
        if (_isSwipingExcessivelyRightFromClosedPosition(gestureState)) {
            return;
        }
        onSwipeStart();
        if (_isSwipingRightFromClosed(gestureState)) {
            _swipeSlowSpeed(gestureState);
        }
        else {
            _swipeFullSpeed(gestureState);
        }
    }, [
        _isSwipingExcessivelyRightFromClosedPosition,
        _isSwipingRightFromClosed,
        _swipeFullSpeed,
        _swipeSlowSpeed,
        onSwipeStart,
    ]);
    // Ignore swipes due to user's finger moving slightly when tapping
    const _isValidSwipe = react_1.useCallback((gestureState) => {
        if (preventSwipeRight &&
            _previousLeft.current === CLOSED_LEFT_POSITION &&
            gestureState.dx > 0) {
            return false;
        }
        return Math.abs(gestureState.dx) > HORIZONTAL_SWIPE_DISTANCE_THRESHOLD;
    }, [_previousLeft, preventSwipeRight]);
    const _handleMoveShouldSetPanResponderCapture = react_1.useCallback((_event, gestureState) => {
        // Decides whether a swipe is responded to by this component or its child
        return gestureState.dy < 10 && _isValidSwipe(gestureState);
    }, [_isValidSwipe]);
    const _onSwipeableViewLayout = (event) => {
        setSwipeableViewRendered(true);
        setRowHeight(event.nativeEvent.layout.height);
    };
    const _animateBounceBack = react_1.useCallback((duration) => {
        /**
         * When swiping right, we want to bounce back past closed position on release
         * so users know they should swipe right to get content.
         */
        const swipeBounceBackDistance = IS_RTL
            ? -RIGHT_SWIPE_BOUNCE_BACK_DISTANCE
            : RIGHT_SWIPE_BOUNCE_BACK_DISTANCE;
        _animateTo(-swipeBounceBackDistance, duration, _animateToClosedPositionDuringBounce);
    }, [_animateTo, _animateToClosedPositionDuringBounce]);
    const _animateToOpenPosition = react_1.useCallback(() => {
        const distance = IS_RTL ? -maxSwipeDistance : maxSwipeDistance;
        _animateTo(-distance);
    }, [_animateTo, maxSwipeDistance]);
    const _handlePanResponderEnd = react_1.useCallback((_event, gestureState) => {
        const horizontalDistance = IS_RTL ? -gestureState.dx : gestureState.dx;
        if (_isSwipingRightFromClosed(gestureState)) {
            onOpen();
            _animateBounceBack(RIGHT_SWIPE_BOUNCE_BACK_DURATION);
        }
        else if (_shouldAnimateRemainder(gestureState)) {
            if (horizontalDistance < 0) {
                // Swiped left
                onOpen();
                _animateToOpenPositionWith(gestureState.vx, horizontalDistance);
            }
            else {
                // Swiped right
                onClose();
                _animateToClosedPosition();
            }
        }
        else {
            if (_previousLeft.current === CLOSED_LEFT_POSITION) {
                _animateToClosedPosition();
            }
            else {
                _animateToOpenPosition();
            }
        }
        onSwipeEnd();
    }, [
        _animateBounceBack,
        _animateToClosedPosition,
        _animateToOpenPosition,
        _animateToOpenPositionWith,
        _isSwipingRightFromClosed,
        _previousLeft,
        _shouldAnimateRemainder,
        onClose,
        onOpen,
        onSwipeEnd,
    ]);
    let slideOutView;
    if (isSwipeableViewRendered && rowHeight) {
        slideOutView = (react_1.default.createElement(react_native_1.View, { style: [styles.slideOutContainer, { height: rowHeight }] }, slideoutView));
    }
    // The swipeable item
    const swipeableView = (react_1.default.createElement(react_native_1.Animated.View, { onLayout: _onSwipeableViewLayout, style: { transform: [{ translateX: currentLeft }] } }, children));
    const _panResponder = react_1.useMemo(() => react_native_1.PanResponder.create({
        onMoveShouldSetPanResponderCapture: _handleMoveShouldSetPanResponderCapture,
        onPanResponderGrant: () => ({}),
        onPanResponderMove: _handlePanResponderMove,
        onPanResponderRelease: _handlePanResponderEnd,
        onPanResponderTerminationRequest: () => false,
        onPanResponderTerminate: _handlePanResponderEnd,
        onShouldBlockNativeResponder: () => false,
    }), [
        _handleMoveShouldSetPanResponderCapture,
        _handlePanResponderEnd,
        _handlePanResponderMove,
    ]);
    react_1.useEffect(() => {
        if (shouldBounceOnMount) {
            /**
             * Do the on mount bounce after a delay because if we animate when other
             * components are loading, the animation will be laggy
             */
            setTimeout(() => {
                _animateBounceBack(ON_MOUNT_BOUNCE_DURATION);
            }, ON_MOUNT_BOUNCE_DELAY);
        }
    }, [_animateBounceBack, shouldBounceOnMount]);
    react_1.useEffect(() => {
        if (!isOpen) {
            _animateToClosedPosition();
        }
        else {
            _animateToOpenPosition();
        }
    }, [_animateToClosedPosition, _animateToOpenPosition, isOpen]);
    return (react_1.default.createElement(react_native_1.View, Object.assign({}, _panResponder.panHandlers),
        slideOutView,
        swipeableView));
};
const styles = react_native_1.StyleSheet.create({
    slideOutContainer: {
        bottom: 0,
        left: 0,
        position: 'absolute',
        right: 0,
        top: 0,
    },
});
